;+
; NAME:	RBSP_LOAD_EFW_ESVY_DSC
;
; SYNTAX:
;   rbsp_load_efw_esvy_dsc,probe='a'
;
; PURPOSE:	Loads EFW ESVY data and despins based on HSK SC_SpinPeriod and
;			SC_SpinPhase
;          
; KEYWORDS:
;	probe = 'a' or 'b'  NOTE: single spacecraft only, does not accept ['a b']
;	delta = degrees, angle between sun sensor and +U (EFW 1) boom
;			(default is 10 degrees, then angle between SSHA and +U)
;	/save_hsk - save all HSK data
;	/save_offset - save the calculated offset in TPLOT vars
;	/save_detrend - save E - E_offset in TPLOT vars
;	/debug - save various quantities for debugging
;
; NOTES:
;	1) The default delta, the angle between the default sun sensor SSHA and the
;		+U (EFW 1) boom, is 10 degrees.  Notes and diagrams from the RBSP SPICE
;		instrument frame kernels, rbsp{a,b}_fk_2012_213.tf, are included at the
;		end of this file.  The basic geometric configuration is this:
;			SSHA is rotated 45 degrees from SC +X towards SC -Y.
;			EFW +U (boom 1) is rotated 35 degrees from SC +X towards SC -Y.
;		This angle will have to be changed if either spacecraft is switched from
;		using the default SSHA to the SSHB sun sensor.
;
;	2) This is designed for ESVY, which is mostly continuous.  The etype var
;		could possibly be set to 'eb1' or 'eb2' in order to despin burst
;		electric field data.  This has not been tested.  It may work, although
;		nothing has been done to handle gaps in the data.  If anyone tries this
;		and can confirm that it works, please convert the etype var to a
;		user-specified keyword (the name of the routine should also be changed
;		accordingly).
;
; HISTORY:
;	1. Created Oct 2012 - Kris Kersten, kris.kersten@gmail.com
;	2. 18 Oct 2012 - minor code cleanup - KK
;
; VERSION:
;   $LastChangedBy: kersten $
;   $LastChangedDate: 2012-10-18 08:29:55 -0700 (Thu, 18 Oct 2012) $
;   $LastChangedRevision: 11047 $
;   $URL: svn+ssh://thmsvn@ambrosia.ssl.berkeley.edu/repos/ssl_general/trunk/missions/rbsp/efw/rbsp_load_efw_esvy_dsc.pro $
;-


pro rbsp_load_efw_esvy_dsc,probe=probe,delta=delta,save_hsk=save_hsk,$
	save_offset=save_offset,save_detrend=save_detrend,debug=debug


	etype='esvy'
	
	; check probe keyword
	if ~keyword_set(probe) then begin
		message,"Probe keyword not set. Using default probe='a'.",/continue
		probe='a'
	endif else begin
		probe=strlowcase(probe) ; this turns any data type to a string
		if probe ne 'a' and probe ne 'b' then begin
			message,"Invalid probe keyword. Using default probe='a'.",/continue
			probe='a'
		endif
	endelse

	
	; see if survey data is loaded
	enames=''
	tplot_names,'rbsp'+probe+'*_'+etype,names=enames
	if enames[0] eq '' then noedata=1b else noedata=0b
	
	; see if spin period and spin phase data is loaded
	snames=''
	tplot_names,'rbsp'+probe+'_efw_hsk_idpu_eng_SC_Spin*',names=snames
	if size(snames,/n_elements) ne 2 then nospindata=1b else nospindata=0b


	; load survey, housekeeping if we're missing E or SC_Spin* data
	if noedata then $
		rbsp_load_efw_waveform, probe=probe, datatype=etype, type='cal'

	get_data,'rbsp'+probe+'_efw_'+etype,data=e,limits=el,dlimits=edl


	; save a list of currently loaded vars
	tplot_names,'*',names=allnames
	
	if nospindata then $
		rbsp_load_efw_hsk,probe=probe,/get_support_data
	tplot_names,'*hsk*',names=hsknames
	
	get_data,'rbsp'+probe+'_efw_hsk_idpu_eng_SC_SpinPeriod',data=speriod,limits=speriodl, $
		dlimits=sperioddl
	get_data,'rbsp'+probe+'_efw_hsk_idpu_eng_SC_SpinPhase',data=sphase,limits=sphasel, $
		dlimits=sphasedl

	; clear out all the extra HSK vars
	if ~keyword_set(save_hsk) then begin
		for i=0,size(hsknames,/n_elements)-1 do begin
			temp=where(allnames eq hsknames[i],nn)
			if nn eq 0 then store_data,hsknames[i],/delete
		endfor
	endif

	; restore the SC_Spin* vars
	store_data,'rbsp'+probe+'_efw_hsk_idpu_eng_SC_SpinPeriod',data=speriod,limits=speriodl, $
		dlimits=sperioddl
	store_data,'rbsp'+probe+'_efw_hsk_idpu_eng_SC_SpinPhase',data=sphase,limits=sphasel, $
		dlimits=sphasedl
	
	; split up the timeseries data
	tsp=speriod.x
	sp=speriod.y
	tsph=sphase.x
	sph=sphase.y
	
	te=e.x
	e12=e.y[*,0]
	e34=e.y[*,1]
	e56=e.y[*,2]


	; tmin,tmax define the range over which we have we have both spin and E data
	tmin=max([min(tsp),min(te)])
	tmax=min([max(tsp),max(te)])


	; define the start and end indices for spin and E
	sstart=min(where(tsp ge tmin))
	send=max(where(tsp le tmax))
	ns=send-sstart
	estart=min(where(te ge tmin))
	eend=max(where(te le tmax))
	nee=eend-estart
	dte=median(te[estart+1:eend]-te[estart:eend-1])


	; offset removal
	;
	; ALGORITHM (boxcar average):
	;	0) Set up range to skip beginning and end edge (1/2 spin, for averaging)
	;	1) Find updated spin period value within current spin
	;	2) Set up range of Esvy points that fall completely within current SP
	;	3) Interpolate at each end so that we have exactly one spin over which
	;		to calculate the mean
	;	4) Calculate the mean of the spin and assign that value to the center
	;		point in the offset var.
	;	5) Slide up one point and repeat until we hit the end of E data

	; ... in pictures
	;
	;xxxxx  <-edgelow                                edgehigh->  xxxxx;
	;       .-.         .-.         .-.    |    | .-.         .-.     ;
	;      /   \       /   \       /   \   |    |/   \       /   \    ;
	;...../.....\...../.....\...../.....\  | // |.....\...../.....\...;
	;\   /       \   /       \   /       \ |    |      \   /       \  ;
	; `-'         `-'         `-'         `|    |       `-'         `-;
	;|----c----|
	; |----c----|
	;  |----c----|  -->>slides through data point-by-point
	;
	;  -a spin-period window of Esvy data slides point-by-point through data.
	;  -midpoint (marked c above) takes on the mean value of each "spin".
	;  -xxxxx mark edgelow, edgehigh, where we set offset to zero
	;

	edgelow=long(sp[sstart]/dte)/2
	edgehigh=long(sp[send]/dte)/2
	
	e12o=dblarr(nee)
	e34o=dblarr(nee)
 
	e12o[0:estart+edgelow]=0
	e12o[eend-edgehigh:nee-1]=0
	e34o[0:estart+edgelow]=0
	e34o[eend-edgehigh:nee-1]=0
	
	nhalf=edgelow
	
	for i=estart+edgelow+1,eend-edgehigh-1 do begin

		; use nhalf from last sp to find new sp, and then update nhalf	
		sptemp=sp[min(where(tsp gt te[i-nhalf]))]
		nhalf=long(sptemp/dte)/2
		
		; find the difference between E data point times and full spin time
		; and set up end padding
		tshort=sptemp-(te[i+nhalf]-te[i-nhalf])
		tpad=tshort/2.d
		
		; interpolate at either end and sandwich the E data so we cover exactly
		; one spin period
		tlow=te[i-nhalf]-tpad
		thigh=te[i+nhalf]+tpad
		e12low=e12[i-nhalf-1]+(te[i-nhalf]-te[i-nhalf-1]-tpad)*(e12[i-nhalf]-e12[i-nhalf-1])/ $
				(te[i-nhalf]-te[i-nhalf-1])
		e12high=e12[i+nhalf]+tpad*(e12[i+nhalf+1]-e12[i+nhalf])/ $
				(te[i+nhalf+1]-te[i+nhalf])

		e34low=e34[i-nhalf-1]+(te[i-nhalf]-te[i-nhalf-1]-tpad)*(e34[i-nhalf]-e34[i-nhalf-1])/ $
				(te[i-nhalf]-te[i-nhalf-1])
		e34high=e34[i+nhalf]+tpad*(e34[i+nhalf+1]-e34[i+nhalf])/ $
				(te[i+nhalf+1]-te[i+nhalf])

		e12temp=[e12low,e12[i-nhalf:i+nhalf],e12high]
		e34temp=[e34low,e34[i-nhalf:i+nhalf],e34high]

		; and average
		e12o[i]=mean(e12temp)
		e34o[i]=mean(e34temp)

	endfor
	
	if keyword_set(save_offset) or keyword_set(debug) then begin
		store_data,'rbsp'+probe+'_efw_esvy_e12offset',data={x:te[estart:eend-1],y:e12o}
		store_data,'rbsp'+probe+'_efw_esvy_e34offset',data={x:te[estart:eend-1],y:e34o}
	endif

	e12detrend=e12[estart:eend-1]-e12o
	e34detrend=e34[estart:eend-1]-e34o

	if keyword_set(save_detrend) or keyword_set(debug) then begin
		store_data,'rbsp'+probe+'_efw_esvy_e12detrend',data={x:te[estart:eend-1],y:e12detrend}
		store_data,'rbsp'+probe+'_efw_esvy_e34detrend',data={x:te[estart:eend-1],y:e34detrend}
	endif



	; despin
	;
	; ALGORITHM:
	;	0) Find two SC_SpinPhase points bracketing the current E field time
	;	1) Find linear rate of change of phase (deg)
	;	2) Interpolate to the time of the e field sample.
	;	3) calculate cos(phase(t) + delta), sin(phase(t) + delta)
	;	4) rotate:
	;		[ ExDSC ] = [ cos( )   -sin( ) ] [ E12detrend ]
	;		[ EyDSC ]   [ sin( )    cos( ) ] [ E34detrend ]
	;
	
	exdsc=dblarr(nee)
	eydsc=dblarr(nee)	
	
	x=dblarr(nee)
	y=dblarr(nee)
	esph=dblarr(nee)
	dsph=dblarr(nee)
	dtsph=dblarr(nee)
	
	; get the sign of median change in spin phase (for handling 0->360 rollover)
	medsph=median(sph[sstart:send])
	if medsph lt 0 then signphase=-1. else signphase=1.

	; check for delta keyword
	if ~keyword_set(delta) then delta=!dtor*10.
	
	for i=0L,nee-1L do begin
		
		isph=min(where(tsph ge te[i+estart]))
		if isph-1 gt sstart then begin
			; handle rollover
			dsph[i]=sph[isph]-sph[isph-1]
			if dsph[i]*signphase lt 0. then dsph[i]=dsph[i]+360.*signphase
			dtsph[i]=tsph[isph]-tsph[isph-1]
		endif else begin
			dsph[i]=sph[isph+1]-sph[isph]
			if dsph[i]*signphase lt 0. then dsph[i]=dsph[i]+360.*signphase
			dtsph[i]=tsph[isph+1]-tsph[isph]
		endelse
		
		esph[i]=sph[isph]+(te[i+estart]-tsph[isph])*dsph[i]/dtsph[i]
		
		; wrap esph
		if esph[i] gt 360. then esph[i]-=360.
		if esph[i] lt 0. then esph[i]+=360.

		; convert to radians
		phi=!dtor*esph[i]
		
		x[i]=cos(phi+delta)
		y[i]=sin(phi+delta)
		
		exdsc[i]=e12detrend[i]*x[i]-e34detrend[i]*y[i]
		eydsc[i]=e12detrend[i]*y[i]+e34detrend[i]*x[i]

	endfor

	if keyword_set(debug) then begin
		store_data,'rbsp'+probe+'_dsph_debug',data={x:te[estart:eend-1],y:dsph}
		store_data,'rbsp'+probe+'_dtsph_debug',data={x:te[estart:eend-1],y:dtsph}
	
		store_data,'rbsp'+probe+'_esph_debug',data={x:te[estart:eend-1],y:esph}
		store_data,'rbsp'+probe+'_DSC_cos_debug',data={x:te[estart:eend-1],y:x}
		store_data,'rbsp'+probe+'_DSC_sin_debug',data={x:te[estart:eend-1],y:y}
	endif
	
	store_data,'rbsp'+probe+'_efw_esvy_exDSC',data={x:te[estart:eend-1],y:exdsc}
	store_data,'rbsp'+probe+'_efw_esvy_eyDSC',data={x:te[estart:eend-1],y:eydsc}

end


; The following notes and diagrams are from the rbsp{a,b}_fk_2012_213.tf SPICE
; frame kernels, which are based on the definitions in "RBSP G&C Coordinate
; Systems, Component Alignments and Naming Conventions", R. M. Vaughan.
;
;
;   Each RBSP spacecraft carries a Sun Sensor (SS) system. The system consists
;   of two Sun Sensor Heads (SSHs). The two SSHs are attached to opposite side
;   panels of the spacecraft near the top deck so that they look out between
;   two of the solar panels. The heads provide a Sun offset angle from a
;   reference axis that is nominally aligned with the spacecraft +Z axis when
;   the Sun crosses through their respective fields-of-view; two measurements
;   (one per head) per spin should be obtained during nominal operation [5,6].
;   The SSH mounted to panel #4 in the +X -Y quadrant of the spacecraft frame
;   is named “SSH A” and the SSH mounted to panel #0 in the -X +Y quadrant is
;   named “SSH B”.
;
;   The SSHs look radially outward in a plane that is perpendicular to the
;   spacecraft XY plane and makes an angle of 45 degrees from the spacecraft XZ
;   plane.
;
;              +Y (spacecraft)
;              /|\
;               |
;               |
;               |
;  +X (SSH B)   |          +Y (SSH A)
;   .__         |         __.
;   |',         |         ,'|    
;      ',     o |       ,'   
;        ', 45  |     ,'  
;          ',   |   ,'  o
;            ', | ,'  45
;              ,O'----------------------------> +X (spacecraft)
;            ,' | ',                            
;          ,'   |   ',           
;        ,'     |     ',     
;   |  ,'       |       ',  |
;   |.'__       |       __'.|
;  +Y (SSH B)   |          +X (SSH A)   
;               |
;               |
;               |
;              \|/
;              -Y (spacecraft)
;
;
;
;
;
;
;
;                                    |____          o
;                  o           .   ' |\'  .       35            .`
;                55       . '        | \     ' .              .`
;                       '            |           '          .`
;                    '       ________|_______       '     .`
;                  '        |        |       |        ' .`___
; .              '          |        |       |        .`\ .
;   ` .     |  '            |        |       |      .`   \  '          o
;       ` . |.___           |        |       |    .`           '     55 
;  o     . '` .             |        |       |  .`               '
;35    .        ` .         |________|_______|.`                   '
;     '             ` .       .------|-----..`                      ' 
;    '   _______________` .  /       |      \     ______________     '
;   '   |               |  /      .-`|-.      \  |              |     '
;  \|/  |               | |      / +Z|  \      | |              |     |
;-------|---------------|-|-----|----O---|-----|-|--------------|--------> +Y
;  |    |               | |      \   |  /      | |              |    /|\(Space-
;  '    |               |  \      `-.|-`      /  |              |        craft)
;  '    '_______________'    \       |      /  ` .______________`     '   o
;   '                       .``------|-----`       ` .               .  35
;    `          EFW Boom  .` ________|_______          ` .              
;  o  `            #1-->.`  |        |       |  EFW Boom-->` .      ' 
;55     '             .`    |        |       |      #3       _`_. /
;         '         .`      |        |       |                   /|` .
;           '  |  .`        |        |       |                 .  |    ` .|
;           ---`.`          |        |       |                          -- 
;             .` '          |________|_______|              '            +Y
;           .`     '                 |                  .             (Science)
;        |.`          '              |               .   o
;        |--           o ' .         |           . '   55
;        +X          35       '  . _\|_ .  -  '
;     (Science)                     /|
;                                    |
;                                   \|/ 
;                                   +X
;                               (Spacecraft)
 